@@ -57,8 +57,29 @@ class TestGeneratesWithFunctionAndClassVariableArg(GeneratesTestCase):
         self.Article = Article
 
 
+class DeepPathGeneratesTestCase(TestCase):
+    def test_simple_dotted_source_path(self):
+        document = self.Document(name=u'Document 1', locale='en')
+        section = self.Section(name=u'Section 1', document=document)
 
-class TestGeneratesWithSourcePath(TestCase):
+        self.session.add(document)
+        self.session.add(section)
+        self.session.commit()
+
+        assert section.locale == 'en'
+
+    def test_deep_dotted_source_path(self):
+        document = self.Document(name=u'Document 1', locale='en')
+        section = self.Section(name=u'Section 1', document=document)
+        subsection = self.SubSection(name=u'Section 1', section=section)
+
+        self.session.add(subsection)
+        self.session.commit()
+
+        assert subsection.locale == 'en'
+
+
+class TestGeneratesWithSourcePath(DeepPathGeneratesTestCase):
     def create_models(self):
         class Document(self.Base):
             __tablename__ = 'document'
@@ -102,23 +123,3 @@ class TestGeneratesWithSourcePath(TestCase):
         self.Document = Document
         self.Section = Section
         self.SubSection = SubSection
-
-    def test_simple_dotted_source_path(self):
-        document = self.Document(name=u'Document 1', locale='en')
-        section = self.Section(name=u'Section 1', document=document)
-
-        self.session.add(document)
-        self.session.add(section)
-        self.session.commit()
-
-        assert section.locale == 'en'
-
-    def test_deep_dotted_source_path(self):
-        document = self.Document(name=u'Document 1', locale='en')
-        section = self.Section(name=u'Section 1', document=document)
-        subsection = self.SubSection(name=u'Section 1', section=section)
-
-        self.session.add(subsection)
-        self.session.commit()
-
-        assert subsection.locale == 'en'
